@@ -28,6 +28,7 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.zip.CRC32;
+import org.h2.api.ErrorCode;
 import org.h2.api.JavaObjectSerializer;
 import org.h2.compress.CompressLZF;
 import org.h2.engine.Constants;
@@ -53,6 +54,7 @@ import org.h2.store.FileStore;
 import org.h2.store.FileStoreInputStream;
 import org.h2.store.LobStorageBackend;
 import org.h2.store.LobStorageFrontend;
+import org.h2.store.LobStorageMap;
 import org.h2.store.Page;
 import org.h2.store.PageFreeList;
 import org.h2.store.PageLog;
@@ -69,6 +71,7 @@ import org.h2.util.StringUtils;
 import org.h2.util.TempFileDeleter;
 import org.h2.util.Tool;
 import org.h2.util.Utils;
+import org.h2.value.CompareMode;
 import org.h2.value.Value;
 import org.h2.value.ValueArray;
 import org.h2.value.ValueLob;
@@ -213,16 +216,26 @@ public class Recover extends Tool implements DataHandler {
     public static Value.ValueBlob readBlobDb(Connection conn, long lobId,
             long precision) {
         DataHandler h = ((JdbcConnection) conn).getSession().getDataHandler();
+        verifyPageStore(h);
         return ValueLobDb.create(Value.BLOB, h, LobStorageFrontend.TABLE_TEMP,
                 lobId, null, precision);
     }
 
+    private static void verifyPageStore(DataHandler h) {
+        if (h.getLobStorage() instanceof LobStorageMap) {
+            throw DbException.get(ErrorCode.FEATURE_NOT_SUPPORTED_1,
+                    "Restore page store recovery SQL script " +
+                    "can only be restored to a PageStore file");
+        }
+    }
+
     /**
      * INTERNAL
      */
     public static Value.ValueClob readClobDb(Connection conn, long lobId,
             long precision) {
         DataHandler h = ((JdbcConnection) conn).getSession().getDataHandler();
+        verifyPageStore(h);
         return ValueLobDb.create(Value.CLOB, h, LobStorageFrontend.TABLE_TEMP,
                 lobId, null, precision);
     }
@@ -683,6 +696,7 @@ public class Recover extends Tool implements DataHandler {
                 "INFORMATION_SCHEMA.LOB_BLOCKS(" +
                 "LOB_ID BIGINT, SEQ INT, DATA BINARY, " +
                 "PRIMARY KEY(LOB_ID, SEQ));");
+        boolean hasErrors = false;
         for (Entry<Long, Object[]> e : lobMap.entrySet()) {
             long lobId = e.getKey();
             Object[] value = e.getValue();
@@ -704,6 +718,22 @@ public class Recover extends Tool implements DataHandler {
                 }
             } catch (IOException ex) {
                 writeError(writer, ex);
+                hasErrors = true;
+            }
+        }
+        writer.println("-- lobMap.size: " + lobMap.sizeAsLong());
+        writer.println("-- lobData.size: " + lobData.sizeAsLong());
+
+        if (hasErrors) {
+            writer.println("-- lobMap");
+            for (Long k : lobMap.keyList()) {
+                Object[] value = lobMap.get(k);
+                byte[] streamStoreId = (byte[]) value[0];
+                writer.println("--     " + k + " " + StreamStore.toString(streamStoreId));
+            }
+            writer.println("-- lobData");
+            for (Long k : lobData.keyList()) {
+                writer.println("--     " + k + " len " + lobData.get(k).length);
             }
         }
     }
@@ -1717,4 +1747,9 @@ public class Recover extends Tool implements DataHandler {
     public JavaObjectSerializer getJavaObjectSerializer() {
         return null;
     }
+
+    @Override
+    public CompareMode getCompareMode() {
+        return CompareMode.getInstance(null, 0);
+    }
 }
