@@ -6,6 +6,9 @@
 package org.h2.test.db;
 
 import java.io.BufferedInputStream;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.File;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
@@ -22,6 +25,7 @@ import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Timestamp;
 import java.sql.Types;
 import java.util.ArrayList;
 import java.util.Calendar;
@@ -42,6 +46,7 @@ import org.h2.test.TestBase;
 import org.h2.tools.SimpleResultSet;
 import org.h2.util.IOUtils;
 import org.h2.util.New;
+import org.h2.util.StringUtils;
 import org.h2.value.Value;
 
 /**
@@ -57,6 +62,8 @@ public class TestFunctions extends TestBase implements AggregateFunction {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
+        // Locale.setDefault(Locale.GERMANY);
+        // Locale.setDefault(Locale.US);
         TestBase.createCaller().init().test();
     }
 
@@ -89,16 +96,15 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         testNvl2();
         testConcatWs();
         testTruncate();
+        testOraHash();
         testToCharFromDateTime();
         testToCharFromNumber();
         testToCharFromText();
         testTranslate();
-
-        // TODO
-        // testCachingOfDeterministicFunctionAlias();
+        testGenerateSeries();
+        testFileWrite();
 
         deleteDb("functions");
-        FileUtils.deleteRecursive(TEMP_DIR, true);
     }
 
     private void testDataType() throws SQLException {
@@ -623,6 +629,36 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         FileUtils.delete(fileName);
     }
 
+
+    private void testFileWrite() throws Exception {
+        Connection conn = getConnection("functions");
+        Statement stat = conn.createStatement();
+        // Copy data into clob table
+        stat.execute("DROP TABLE TEST IF EXISTS");
+        PreparedStatement pst = conn.prepareStatement(
+                "CREATE TABLE TEST(data clob) AS SELECT ? " + "data");
+        Properties prop = System.getProperties();
+        ByteArrayOutputStream os = new ByteArrayOutputStream(prop.size());
+        prop.store(os, "");
+        pst.setBinaryStream(1, new ByteArrayInputStream(os.toByteArray()));
+        pst.execute();
+        os.close();
+        String fileName = new File(getBaseDir(), "test.txt").getPath();
+        FileUtils.delete(fileName);
+        ResultSet rs = stat.executeQuery("SELECT FILE_WRITE(data, " +
+                StringUtils.quoteStringSQL(fileName) + ") len from test");
+        assertTrue(rs.next());
+        assertEquals(os.size(), rs.getInt(1));
+        InputStreamReader r = new InputStreamReader(FileUtils.newInputStream(fileName));
+        // Compare expected content with written file content
+        String ps2 = IOUtils.readStringAndClose(r, -1);
+        assertEquals(os.toString(), ps2);
+        conn.close();
+        FileUtils.delete(fileName);
+    }
+
+
+
     /**
      * This median implementation keeps all objects in memory.
      */
@@ -1223,6 +1259,19 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         conn.close();
     }
 
+    private void testOraHash() throws SQLException {
+        deleteDb("functions");
+        Connection conn = getConnection("functions");
+        Statement stat = conn.createStatement();
+        String testStr = "foo";
+        assertResult(String.valueOf("foo".hashCode()), stat,
+                String.format("SELECT ORA_HASH('%s') FROM DUAL", testStr));
+        assertResult(String.valueOf("foo".hashCode()), stat,
+                String.format("SELECT ORA_HASH('%s', 0) FROM DUAL", testStr));
+        assertResult(String.valueOf("foo".hashCode()), stat,
+                String.format("SELECT ORA_HASH('%s', 0, 0) FROM DUAL", testStr));
+    }
+
     private void testToCharFromDateTime() throws SQLException {
         deleteDb("functions");
         Connection conn = getConnection("functions");
@@ -1242,7 +1291,10 @@ public class TestFunctions extends TestBase implements AggregateFunction {
 
         assertResult("1979-11-12 08:12:34.56", stat, "SELECT X FROM T");
         assertResult("-100-01-15 14:04:02.12", stat, "SELECT X FROM U");
-        assertResult("12-NOV-79 08.12.34.560000 AM", stat, "SELECT TO_CHAR(X) FROM T");
+        String expected = String.format("%tb",
+                Timestamp.valueOf("1979-11-12 08:12:34.560")).toUpperCase();
+        assertResult("12-" + expected + "-79 08.12.34.560000 AM", stat,
+                "SELECT TO_CHAR(X) FROM T");
         assertResult("- / , . ; : text - /", stat,
                 "SELECT TO_CHAR(X, '- / , . ; : \"text\" - /') FROM T");
         assertResult("1979-11-12", stat,
@@ -1289,30 +1341,40 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         assertResult("am", stat, "SELECT TO_CHAR(X, 'pm') FROM T");
         assertResult("2", stat, "SELECT TO_CHAR(X, 'D') FROM T");
         assertResult("2", stat, "SELECT TO_CHAR(X, 'd') FROM T");
-        assertResult("MONDAY   ", stat, "SELECT TO_CHAR(X, 'DAY') FROM T");
-        assertResult("Monday   ", stat, "SELECT TO_CHAR(X, 'Day') FROM T");
-        assertResult("monday   ", stat, "SELECT TO_CHAR(X, 'day') FROM T");
-        assertResult("monday   ", stat, "SELECT TO_CHAR(X, 'dAY') FROM T");
-        assertResult("Monday", stat, "SELECT TO_CHAR(X, 'fmDay') FROM T");
-        assertResult("monday   -monday-monday-monday   -monday", stat,
-                "SELECT TO_CHAR(X, 'day-fmday-day-fmday-fmday') FROM T");
+        expected = String.format("%tA",
+                Timestamp.valueOf("1979-11-12 08:12:34.560"));
+        expected = expected.substring(0, 1).toUpperCase() + expected.substring(1);
+        String spaces = "         ";
+        String first9 = (expected + spaces).substring(0, 9);
+        assertResult(first9.toUpperCase(),
+                stat, "SELECT TO_CHAR(X, 'DAY') FROM T");
+        assertResult(first9,
+                stat, "SELECT TO_CHAR(X, 'Day') FROM T");
+        assertResult(first9.toLowerCase(),
+                stat, "SELECT TO_CHAR(X, 'day') FROM T");
+        assertResult(first9.toLowerCase(),
+                stat, "SELECT TO_CHAR(X, 'dAY') FROM T");
+        assertResult(expected,
+                stat, "SELECT TO_CHAR(X, 'fmDay') FROM T");
         assertResult("12", stat, "SELECT TO_CHAR(X, 'DD') FROM T");
         assertResult("316", stat, "SELECT TO_CHAR(X, 'DDD') FROM T");
         assertResult("316", stat, "SELECT TO_CHAR(X, 'DdD') FROM T");
         assertResult("316", stat, "SELECT TO_CHAR(X, 'dDD') FROM T");
         assertResult("316", stat, "SELECT TO_CHAR(X, 'ddd') FROM T");
-        assertResult("Monday, November 12, 1979", stat,
+        expected = String.format("%1$tA, %1$tB %1$te, %1$tY",
+                Timestamp.valueOf("1979-11-12 08:12:34.560"));
+        assertResult(expected, stat,
                 "SELECT TO_CHAR(X, 'DL') FROM T");
-        assertResult("Monday, November 12, 1979", stat,
-                "SELECT TO_CHAR(X, 'DL', 'NLS_DATE_LANGUAGE = English') FROM T");
         assertResult("11/12/1979", stat, "SELECT TO_CHAR(X, 'DS') FROM T");
         assertResult("11/12/1979", stat, "SELECT TO_CHAR(X, 'Ds') FROM T");
         assertResult("11/12/1979", stat, "SELECT TO_CHAR(X, 'dS') FROM T");
         assertResult("11/12/1979", stat, "SELECT TO_CHAR(X, 'ds') FROM T");
-        assertResult("MON", stat, "SELECT TO_CHAR(X, 'DY') FROM T");
-        assertResult("Mon", stat, "SELECT TO_CHAR(X, 'Dy') FROM T");
-        assertResult("mon", stat, "SELECT TO_CHAR(X, 'dy') FROM T");
-        assertResult("mon", stat, "SELECT TO_CHAR(X, 'dY') FROM T");
+        expected = String.format("%1$ta",
+                Timestamp.valueOf("1979-11-12 08:12:34.560"));
+        assertResult(expected.toUpperCase(), stat, "SELECT TO_CHAR(X, 'DY') FROM T");
+        assertResult(expected, stat, "SELECT TO_CHAR(X, 'Dy') FROM T");
+        assertResult(expected.toLowerCase(), stat, "SELECT TO_CHAR(X, 'dy') FROM T");
+        assertResult(expected.toLowerCase(), stat, "SELECT TO_CHAR(X, 'dY') FROM T");
         assertResult("08:12:34.560000", stat,
                 "SELECT TO_CHAR(X, 'HH:MI:SS.FF') FROM T");
         assertResult("08:12:34.5", stat,
@@ -1353,13 +1415,26 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         assertResult("11", stat, "SELECT TO_CHAR(X, 'Mm') FROM T");
         assertResult("11", stat, "SELECT TO_CHAR(X, 'mM') FROM T");
         assertResult("11", stat, "SELECT TO_CHAR(X, 'mm') FROM T");
-        assertResult("NOV", stat, "SELECT TO_CHAR(X, 'MON') FROM T");
-        assertResult("Nov", stat, "SELECT TO_CHAR(X, 'Mon') FROM T");
-        assertResult("nov", stat, "SELECT TO_CHAR(X, 'mon') FROM T");
-        assertResult("NOVEMBER ", stat, "SELECT TO_CHAR(X, 'MONTH') FROM T");
-        assertResult("November ", stat, "SELECT TO_CHAR(X, 'Month') FROM T");
-        assertResult("november ", stat, "SELECT TO_CHAR(X, 'month') FROM T");
-        assertResult("November", stat, "SELECT TO_CHAR(X, 'fmMonth') FROM T");
+        expected = String.format("%1$tb",
+                Timestamp.valueOf("1979-11-12 08:12:34.560"));
+        expected = expected.substring(0, 1).toUpperCase() + expected.substring(1);
+        assertResult(expected.toUpperCase(), stat,
+                "SELECT TO_CHAR(X, 'MON') FROM T");
+        assertResult(expected, stat,
+                "SELECT TO_CHAR(X, 'Mon') FROM T");
+        assertResult(expected.toLowerCase(), stat,
+                "SELECT TO_CHAR(X, 'mon') FROM T");
+        expected = String.format("%1$tB",
+                Timestamp.valueOf("1979-11-12 08:12:34.560"));
+        expected = (expected + "        ").substring(0, 9);
+        assertResult(expected.toUpperCase(), stat,
+                "SELECT TO_CHAR(X, 'MONTH') FROM T");
+        assertResult(expected, stat,
+                "SELECT TO_CHAR(X, 'Month') FROM T");
+        assertResult(expected.toLowerCase(), stat,
+                "SELECT TO_CHAR(X, 'month') FROM T");
+        assertResult(expected.trim(), stat,
+                "SELECT TO_CHAR(X, 'fmMonth') FROM T");
         assertResult("4", stat, "SELECT TO_CHAR(X, 'Q') FROM T");
         assertResult("XI", stat, "SELECT TO_CHAR(X, 'RM') FROM T");
         assertResult("xi", stat, "SELECT TO_CHAR(X, 'rm') FROM T");
@@ -1371,8 +1446,10 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         assertResult("8:12:34 AM", stat, "SELECT TO_CHAR(X, 'TS') FROM T");
         assertResult(tzLongName, stat, "SELECT TO_CHAR(X, 'TZR') FROM T");
         assertResult(tzShortName, stat, "SELECT TO_CHAR(X, 'TZD') FROM T");
-        assertResult(".", stat, "SELECT TO_CHAR(X, 'X') FROM T");
-        assertResult("1,979", stat, "SELECT TO_CHAR(X, 'Y,YYY') FROM T");
+        expected = String.format("%f", 1.1).substring(1, 2);
+        assertResult(expected, stat, "SELECT TO_CHAR(X, 'X') FROM T");
+        expected = String.format("%,d", 1979);
+        assertResult(expected, stat, "SELECT TO_CHAR(X, 'Y,YYY') FROM T");
         assertResult("1979", stat, "SELECT TO_CHAR(X, 'YYYY') FROM T");
         assertResult("1979", stat, "SELECT TO_CHAR(X, 'SYYYY') FROM T");
         assertResult("-0100", stat, "SELECT TO_CHAR(X, 'SYYYY') FROM U");
@@ -1380,7 +1457,8 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         assertResult("79", stat, "SELECT TO_CHAR(X, 'YY') FROM T");
         assertResult("9", stat, "SELECT TO_CHAR(X, 'Y') FROM T");
         assertResult("7979", stat, "SELECT TO_CHAR(X, 'yyfxyy') FROM T");
-        assertThrows("", stat, "SELECT TO_CHAR(X, 'A') FROM T");
+        assertThrows(ErrorCode.INVALID_TO_CHAR_FORMAT, stat,
+                "SELECT TO_CHAR(X, 'A') FROM T");
 
         // check a bug we had when the month or day of the month is 1 digit
         stat.executeUpdate("TRUNCATE TABLE T");
@@ -1435,7 +1513,8 @@ public class TestFunctions extends TestBase implements AggregateFunction {
                 "SELECT TO_CHAR(12345, '$99999999') FROM DUAL");
         assertResult("     " + cs + "12,345.35", stat,
                 "SELECT TO_CHAR(12345.345, '$99,999,999.99') FROM DUAL");
-        assertResult("     " + cs + "12,345", stat,
+        String expected = String.format("%,d", 12345);
+        assertResult("     " + cs + expected, stat,
                 "SELECT TO_CHAR(12345.345, '$99g999g999') FROM DUAL");
         assertResult("     12,345.35", stat,
                 "SELECT TO_CHAR(12345.345, '99,999,999.99') FROM DUAL");
@@ -1535,7 +1614,8 @@ public class TestFunctions extends TestBase implements AggregateFunction {
                 "SELECT TO_CHAR(123.45, 'U999.99') FROM DUAL");
         assertResult("          " + cs + "123.45", stat,
                 "SELECT TO_CHAR(123.45, 'u999.99') FROM DUAL");
-        assertResult("   .33", stat,
+        expected = String.format("%.2f", 0.33f).substring(1);
+        assertResult("   " + expected, stat,
                 "SELECT TO_CHAR(0.326, '99D99') FROM DUAL");
         assertResult("  1.2E+02", stat,
                 "SELECT TO_CHAR(123.456, '9.9EEEE') FROM DUAL");
@@ -1550,9 +1630,10 @@ public class TestFunctions extends TestBase implements AggregateFunction {
                 "SELECT TO_CHAR(123.456, '00.00000000EEEE') FROM DUAL");
         assertResult("1.23456000E+02", stat,
                 "SELECT TO_CHAR(123.456, 'fm00.00000000EEEE') FROM DUAL");
-        assertResult(" 1,234,567", stat,
+        expected = String.format("%,d", 1234567);
+        assertResult(" " + expected, stat,
                 "SELECT TO_CHAR(1234567, '9G999G999') FROM DUAL");
-        assertResult("-1,234,567", stat,
+        assertResult("-" + expected, stat,
                 "SELECT TO_CHAR(-1234567, '9G999G999') FROM DUAL");
         assertResult("123.45-", stat, "SELECT TO_CHAR(-123.45, '999.99MI') FROM DUAL");
         assertResult("123.45-", stat, "SELECT TO_CHAR(-123.45, '999.99mi') FROM DUAL");
@@ -1611,11 +1692,11 @@ public class TestFunctions extends TestBase implements AggregateFunction {
                 "SELECT TO_CHAR(123456789012345, 'TME') FROM DUAL");
         assertResult("4.5E-01", stat, "SELECT TO_CHAR(0.45, 'TME') FROM DUAL");
         assertResult("4.5E-01", stat, "SELECT TO_CHAR(0.45, 'tMe') FROM DUAL");
-        assertThrows("Invalid TO_CHAR format \"999.99q\"", stat,
+        assertThrows(ErrorCode.INVALID_TO_CHAR_FORMAT, stat,
                 "SELECT TO_CHAR(123.45, '999.99q') FROM DUAL");
-        assertThrows("Invalid TO_CHAR format \"fm999.99q\"", stat,
+        assertThrows(ErrorCode.INVALID_TO_CHAR_FORMAT, stat,
                 "SELECT TO_CHAR(123.45, 'fm999.99q') FROM DUAL");
-        assertThrows("Invalid TO_CHAR format \"q999.99\"", stat,
+        assertThrows(ErrorCode.INVALID_TO_CHAR_FORMAT, stat,
                 "SELECT TO_CHAR(123.45, 'q999.99') FROM DUAL");
 
         conn.close();
@@ -1629,6 +1710,56 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         conn.close();
     }
 
+    private void testGenerateSeries() throws SQLException {
+        Connection conn = getConnection("functions");
+        Statement stat = conn.createStatement();
+
+        ResultSet rs = stat.executeQuery("select * from system_range(1,3)");
+        rs.next();
+        assertEquals(1, rs.getInt(1));
+        rs.next();
+        assertEquals(2, rs.getInt(1));
+        rs.next();
+        assertEquals(3, rs.getInt(1));
+
+        rs = stat.executeQuery("select * from system_range(2,2)");
+        assertTrue(rs.next());
+        assertEquals(2, rs.getInt(1));
+
+        rs = stat.executeQuery("select * from system_range(2,1)");
+        assertFalse(rs.next());
+
+        rs = stat.executeQuery("select * from system_range(1,2,-1)");
+        assertFalse(rs.next());
+
+        assertThrows(ErrorCode.STEP_SIZE_MUST_NOT_BE_ZERO, stat).executeQuery(
+                "select * from system_range(1,2,0)");
+
+        rs = stat.executeQuery("select * from system_range(2,1,-1)");
+        assertTrue(rs.next());
+        assertEquals(2, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+
+        rs = stat.executeQuery("select * from system_range(1,5,2)");
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(3, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(5, rs.getInt(1));
+
+        rs = stat.executeQuery("select * from system_range(1,6,2)");
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(3, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(5, rs.getInt(1));
+
+        conn.close();
+    }
+
     private void assertCallResult(String expected, Statement stat, String sql)
             throws SQLException {
         ResultSet rs = stat.executeQuery("CALL " + sql);
